--[[
    Author: Alternator (Massiner of Nathrezim)
    Date:	2014

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local Methods = Engine.Methods;

local C = Engine.Constants;
local S = Engine.Settings;
local Core = Engine.Core;
local Util = Engine.Util;
local Events = Engine.Events;
local CustomActionRegistry = Engine.CustomActionRegistry;
local SecureManagement = Engine.SecureManagement;
local FlyoutUI = Engine.FlyoutUI;
local Scripts = Engine.Scripts;
local ScriptsSpell = Engine.ScriptsSpell;
local ScriptsItem = Engine.ScriptsItem;
local ScriptsMacro = Engine.ScriptsMacro;
local ScriptsMount = Engine.ScriptsMount;
local ScriptsMountFavorite = Engine.ScriptsMountFavorite;
local ScriptsBattlePet = Engine.ScriptsBattlePet;
local ScriptsBattlePetFavorite = Engine.ScriptsBattlePetFavorite;
local ScriptsFlyout = Engine.ScriptsFlyout;
local ScriptsEquipmentSet = Engine.ScriptsEquipmentSet;
local ScriptsCustom = Engine.ScriptsCustom;



--[[------------------------------------------------
    SetAction
--------------------------------------------------
function Methods.SetStoredState(BFButton, StoredState)
    if (StoredState == nil) then
        StoredState = {};
    end
    Util.TableAddUnsetKeys(StoredState, S.ButtonDefaults);
    BFButton.StoredState = StoredState;
    
    Core.ReportEvent(BFButton, C.EVENT_STOREDSTATE);
    
    Methods.SetAction(BFButton, StoredState.Action);
    Methods.SetFlyoutDirection(BFButton, StoredState.FlyoutDirection);
    Methods.SetKeyBindText(BFButton, StoredState.KeyBindText);
    Methods.SetEnabled(BFButton, StoredState.Enabled);
    Methods.SetRespondsToMouse(BFButton, StoredState.RespondsToMouse);
    Methods.SetAlpha(BFButton, StoredState.Alpha);
    Methods.SetLocked(BFButton, StoredState.Locked);
    Methods.SetAlwaysShowGrid(BFButton, StoredState.AlwaysShowGrid);
    Methods.SetShowTooltip(BFButton, StoredState.ShowTooltip);
    Methods.SetShowKeyBindText(BFButton, StoredState.ShowKeyBindText);
    Methods.SetShowCounts(BFButton, StoredState.ShowCounts);
    Methods.SetShowMacroName(BFButton, StoredState.ShowMacroName);

end
function Methods.GetStoredState(BFButton)
    return BFButton.StoredState;
end
]]

--[[------------------------------------------------
    Set/GetFlyoutDirection
    Notes:
        * Non Combat Only
--------------------------------------------------]]
function Methods.SetFlyoutDirection(BFButton, Direction)
    Direction = Direction or "UP";
    BFButton.FlyoutDirection = Direction;
    if (BFButton.ABW:GetAttribute("flyoutDirection") ~= Direction) then
        BFButton.ABW:SetAttribute("flyoutDirection", Direction);
        SpellFlyout:Hide();
        if (BFButton.Type == "flyout") then
            BFButton:FullUpdate();
        end
    end

    Core.ReportEvent(BFButton, C.EVENT_FLYOUTDIRECTION, Direction);
end
function Methods.GetFlyoutDirection(BFButton)
    return BFButton.FlyoutDirection;
end



--[[------------------------------------------------
    Set/GetKeyBindText
--------------------------------------------------]]
function Methods.SetKeyBindText(BFButton, KeyBindText)
    local HotKey = BFButton.ABWHotKey;
    BFButton.KeyBindText = KeyBindText;
    if (KeyBindText == "" or KeyBindText == nil) then
        HotKey:SetText(RANGE_INDICATOR);
        HotKey:Hide();
    else
        HotKey:SetText(KeyBindText);
        HotKey:Show();
    end
    
    BFButton:UpdateRangeRegistration();

    Core.ReportEvent(BFButton, C.EVENT_KEYBINDTEXT, KeyBindText);
    
end
function Methods.GetKeyBindText(BFButton)
    return BFButton.KeyBindText;
end


--[[------------------------------------------------
    Set/GetEnabled
    Notes:
        * Non Combat Only
--------------------------------------------------]]
function Methods.SetEnabled(BFButton, Enabled)
    BFButton.Enabled = Enabled;
    
    if (Enabled) then
        Core.UpdateButtonShowHide(BFButton);
        BFButton:FullUpdate();
    else
        Core.ReleaseTempButtonResources(BFButton);
        BFButton.ABW:Hide();
    end
    --[[
    if (BFButton.Enabled and (BFButton.AlwaysShowGrid or BFButton.Type == "empty")) then
        BFButton.ABW:Show();
        BFButton:FullUpdate();
    else
        Core.ReleaseTempButtonResources(BFButton);
        BFButton.ABW:Hide();
    end
    ]]
    
    Core.ReportEvent(BFButton, C.EVENT_ENABLED, Enabled);

end
function Methods.GetEnabled(BFButton)
    return BFButton.Enabled;
end


--[[------------------------------------------------
    Set/GetRespondsToMouse
    Notes:
        * Non Combat Only
--------------------------------------------------]]
function Methods.SetRespondsToMouse(BFButton, RespondsToMouse)
    BFButton.RespondsToMouse = RespondsToMouse;
    BFButton.ABW:EnableMouse(RespondsToMouse);
    
    Core.ReportEvent(BFButton, C.EVENT_RESPONDSTOMOUSE, RespondsToMouse);
    
end
function Methods.SetRespondsToMouse(BFButton)
    return BFButton.RespondsToMouse;
end


--[[------------------------------------------------
    Set/GetAlpha
--------------------------------------------------]]
function Methods.SetAlpha(BFButton, Alpha)
    BFButton.Alpha = Alpha;	
    BFButton.ABW:SetAlpha(Alpha);
    Core.ReportEvent(BFButton, C.EVENT_ALPHA, Alpha);

end
function Methods.GetAlpha(BFButton)
    return BFButton.Alpha;
end


--[[------------------------------------------------
    Set/GetLocked
--------------------------------------------------]]
function Methods.SetLocked(BFButton, Locked)
    BFButton.Locked = Locked;
    BFButton.ABW:SetAttribute("Locked", Locked);
    Core.ReportEvent(BFButton, C.EVENT_LOCKED, Locked);
end
function Methods.GetLocked(BFButton)
    return BFButton.Locked;
end


--[[------------------------------------------------
    Set/GetFullLock
--------------------------------------------------]]
function Methods.SetFullLock(BFButton, FullLock)
    BFButton.FullLock = FullLock;
    BFButton.ABW:SetAttribute("FullLock", FullLock);
    Core.ReportEvent(BFButton, C.EVENT_FULLLOCK, FullLock);
end
function Methods.GetFullLock(BFButton)
    return BFButton.FullLock;
end


--[[------------------------------------------------
    Set/GetSource
--------------------------------------------------]]
function Methods.SetSource(BFButton, Source)
    BFButton.Source = Source;
    BFButton.ABW:SetAttribute("Source", Source);
    Core.ReportEvent(BFButton, C.EVENT_SOURCE, Source);
end
function Methods.GetSource(BFButton)
    return BFButton.Source;
end
    

--[[------------------------------------------------
    Set/GetAlwaysShowGrid
    Notes:
        * Non Combat Only
--------------------------------------------------]]
function Methods.SetAlwaysShowGrid(BFButton, AlwaysShowGrid)
    BFButton.AlwaysShowGrid = AlwaysShowGrid;
    
    Core.UpdateButtonShowHide(BFButton);
    
    --[[
    if (BFButton.Enabled and (AlwaysShowGrid or BFButton.Type ~= "empty")) then
        BFButton.ABW:Show();
    else
        BFButton.ABW:Hide();
    end
    Core.UpdateOnCombatHideRegistration(BFButton);
    ]]
    Core.ReportEvent(BFButton, C.EVENT_ALWAYSSHOWGRID, AlwaysShowGrid);
end
function Methods.GetAlwaysShowGrid(BFButton)
    return BFButton.AlwaysShowGrid;
end


--[[------------------------------------------------
    Set/GetShowTooltip
--------------------------------------------------]]
function Methods.SetShowTooltip(BFButton, ShowTooltip)
    BFButton.ShowTooltip = ShowTooltip;
    Core.ReportEvent(BFButton, C.EVENT_SHOWTOOLTIP, ShowTooltip);
end
function Methods.GetShowTooltip(BFButton)
    return BFButton.ShowTooltip;
end


--[[------------------------------------------------
    Set/GetShowKeyBindText
--------------------------------------------------]]
function Methods.SetShowKeyBindText(BFButton, ShowKeyBindText)
    BFButton.ShowKeyBindText = ShowKeyBindText;
    Core.ReportEvent(BFButton, C.EVENT_SHOWKEYBINDTEXT, ShowKeyBindText);
end
function Methods.GetShowKeyBindText(BFButton)
    return BFButton.ShowKeyBindText;
end


--[[------------------------------------------------
    Set/GetShowCounts
--------------------------------------------------]]
function Methods.SetShowCounts(BFButton, ShowCounts)
    BFButton.ShowCounts = ShowCounts;
    Core.ReportEvent(BFButton, C.EVENT_SHOWCOUNTS, ShowCounts);
end
function Methods.GetShowCounts(BFButton)
    return BFButton.ShowCounts;
end


--[[------------------------------------------------
    Set/GetShowMacroName
--------------------------------------------------]]
function Methods.SetShowMacroName(BFButton, ShowMacroName)
    BFButton.ShowMacroName = ShowMacroName;
    Core.ReportEvent(BFButton, C.EVENT_SHOWMACRONAME, ShowMacroName);
end
function Methods.GetShowMacroName(BFButton)
    return BFButton.ShowMacroName;
end

    
--[[------------------------------------------------
    SetAction
--------------------------------------------------]]
function Methods.SetAction(BFButton, Action, LinkActionTable)
    local Type = Action.Type;
    if (LinkActionTable) then
        BFButton.OldAction = BFButton.Action; -- A bit hacky, but expedient way to report old action in the events
        BFButton.Action = Action;
    end
    
    if		(Type == "spell")				then return Methods.SetActionSpell(			BFButton, Action.SpellID);
    elseif	(Type == "item")				then return Methods.SetActionItem(				BFButton, Action.ItemID, Action.ItemLink);
    elseif	(Type == "macro")				then return Methods.SetActionMacro(			BFButton, Action.MacroIndex, Action.MacroName, Action.MacroBody);
    elseif	(Type == "mount")				then return Methods.SetActionMount(			BFButton, Action.MountID);
    elseif	(Type == "favoritemount")		then return Methods.SetActionMountFavorite(			BFButton);
    elseif	(Type == "battlepet")			then return Methods.SetActionBattlePet(		BFButton, Action.BattlePetGUID, Action.BattlePetSpeciesName, Action.BattlePetName);
    elseif	(Type == "favoritebattlepet")	then return Methods.SetActionBattlePetFavorite(BFButton);
    elseif	(Type == "flyout")				then return Methods.SetActionFlyout(			BFButton, Action.FlyoutID);
    elseif	(Type == "equipmentset")		then return Methods.SetActionEquipmentSet(		BFButton, Action.EquipmentSetName, Action.EquipmentSetID);
    elseif	(Type == "custom")				then return Methods.SetActionCustom(		BFButton, Action.CustomName, Action.CustomData);
    elseif	(Type == "empty")				then return Methods.SetActionEmpty(			BFButton);
    elseif	(Type == nil)					then return Methods.SetActionEmpty(			BFButton);
    else										 return C.ERROR_UNRECOGNISED_ACTION;
    end
end
function Methods.GetAction(BFButton)
    return BFButton.Action;
end


--[[------------------------------------------------
    SetActionEmpty
--------------------------------------------------]]
function Methods.SetActionEmpty(BFButton)
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "empty") then

        BFButton.UpdateAction				= Scripts.EmptyF;
        BFButton.FullUpdate					= Scripts.EmptyF;
        BFButton.UpdateIcon					= Scripts.EmptyF;
        BFButton.UpdateChecked				= Scripts.EmptyChecked;
        BFButton.UpdateUsable				= Scripts.EmptyF;
        BFButton.UpdateCooldown				= Scripts.EmptyF;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= Scripts.EmptyF;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
        BFButton.ABW.UpdateTooltip			= Scripts.EmptyF;
        
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "empty";
        
        Core.ResetDisplayState(BFButton);
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end
    
    local Action = BFButton.Action;
    Util.ClearTable(Action);
    Action.Type = "empty";
    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);
    
end


--[[------------------------------------------------
    Set Spell
    Notes:
        * WoW v7 (13-Aug-2016)
            * Some Spells can share the same name
                * E.g. Shaman Hex's
            * Talents do not seem to be castable from SpellID
--------------------------------------------------]]
function Methods.SetActionSpell(BFButton, SpellID)

    if (not SpellID) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "spell") then
    
        BFButton.UpdateAction				= ScriptsSpell.UpdateAction;
        BFButton.FullUpdate					= ScriptsSpell.FullUpdate;
        BFButton.UpdateIcon					= ScriptsSpell.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsSpell.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsSpell.UpdateUsable;
        BFButton.UpdateCooldown 			= ScriptsSpell.UpdateCooldown;
        BFButton.UpdateEquipped 			= Scripts.EmptyF;
        BFButton.UpdateText					= ScriptsSpell.UpdateText;
        BFButton.UpdateGlow					= ScriptsSpell.UpdateGlow;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsSpell.GetCursor;
        BFButton.UpdateFlashRegistration	= ScriptsSpell.UpdateFlashRegistration;
        BFButton.UpdateRangeRegistration	= ScriptsSpell.UpdateRangeRegistration;
        BFButton.CheckRange					= ScriptsSpell.CheckRange;
        BFButton.ABW.UpdateTooltip			= ScriptsSpell.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;

        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "spell";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);

    end

    SpellID = FindBaseSpellByID(SpellID);

    BFButton.SpellID		= SpellID;
    BFButton.SpellFullName	= Util.SpellFullName(SpellID);
    BFButton.Target			= "target";

    if (not InCombatLockdown()) then
        ScriptsSpell.UpdateAttributes(BFButton);
        ScriptsSpell.UpdateSecureCursor(BFButton);
    end
    ScriptsSpell.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

end


--[[------------------------------------------------
    Set Item
--------------------------------------------------]]
function Methods.SetActionItem(BFButton, ItemID, ItemLink)
    
    if (not ItemID) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "item") then
    
        BFButton.UpdateAction				= ScriptsItem.UpdateAction;
        BFButton.FullUpdate					= ScriptsItem.FullUpdate;
        BFButton.UpdateIcon					= Scripts.EmptyF;
        BFButton.UpdateChecked				= ScriptsItem.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsItem.UpdateUsable;
        BFButton.UpdateCooldown				= ScriptsItem.UpdateCooldown;
        BFButton.UpdateEquipped				= ScriptsItem.UpdateEquipped;
        BFButton.UpdateText					= ScriptsItem.UpdateText;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsItem.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= ScriptsItem.UpdateRangeRegistration;
        BFButton.CheckRange					= ScriptsItem.CheckRange;
        BFButton.ABW.UpdateTooltip			= ScriptsItem.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;

        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "item";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);

    end
   
    local ItemName, BaseItemLink = GetItemInfo(ItemID);
    if (not ItemName) then
        Events.WatchForItemInfoEvent(ItemID);
    end

    BFButton.ItemID		= ItemID;
    BFButton.ItemName	= ItemName;
    BFButton.IsToy      = C_ToyBox.GetToyInfo(BFButton.ItemID) ~= nil;

    if (not ItemLink or strmatch(ItemLink, ":(%d+)") ~= ItemID) then
        BFButton.ItemLink = BaseItemLink;
    else
        BFButton.ItemLink = ItemLink;
    end

    if (not InCombatLockdown()) then
        ScriptsItem.UpdateAttributes(BFButton);
        ScriptsItem.UpdateSecureCursor(BFButton);
    end
    ScriptsItem.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

end


--[[------------------------------------------------
    Set Macro
--------------------------------------------------]]
function Methods.SetActionMacro(BFButton, MacroIndex, MacroName, MacroBody)
    
    if (not (MacroIndex and MacroName and MacroBody)) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "macro") then
    
        BFButton.UpdateAction				= ScriptsMacro.UpdateAction;
        BFButton.FullUpdate					= ScriptsMacro.FullUpdate;
        BFButton.UpdateIcon					= ScriptsMacro.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsMacro.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsMacro.UpdateUsable;
        BFButton.UpdateCooldown				= ScriptsMacro.UpdateCooldown;
        BFButton.UpdateEquipped				= ScriptsMacro.UpdateEquipped;
        BFButton.UpdateText					= ScriptsMacro.UpdateText;
        BFButton.UpdateGlow					= ScriptsMacro.UpdateGlow;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= ScriptsMacro.UpdateMacro;
        BFButton.GetCursor					= ScriptsMacro.GetCursor;
        BFButton.UpdateFlashRegistration	= ScriptsMacro.UpdateFlashRegistration;
        BFButton.UpdateRangeRegistration	= ScriptsMacro.UpdateRangeRegistration;
        BFButton.CheckRange					= ScriptsMacro.CheckRange;
        BFButton.ABW.UpdateTooltip			= ScriptsMacro.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
        
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "macro";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end

    BFButton.MacroIndex	= MacroIndex;
    BFButton.MacroName	= MacroName;
    BFButton.MacroBody	= MacroBody;

    if (not InCombatLockdown()) then
        ScriptsMacro.UpdateAttributes(BFButton);
        ScriptsMacro.UpdateSecureCursor(BFButton);
    end
    ScriptsMacro.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

end


--[[------------------------------------------------
    Set Mount
--------------------------------------------------]]
function Methods.SetActionMount(BFButton, MountID)
    
    if (not MountID) then
        return Methods.SetActionEmpty(BFButton);
    end
    
    if (MountID == C.SUMMON_RANDOM_FAVORITE_MOUNT_ID) then
        -- It's the random favorite button, use a different code path
        return Methods.SetActionMountFavorite(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "mount") then
    
        BFButton.UpdateAction				= ScriptsMount.UpdateAction;
        BFButton.FullUpdate					= ScriptsMount.FullUpdate;
        BFButton.UpdateIcon					= ScriptsMount.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsMount.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsMount.UpdateUsable;
        BFButton.UpdateCooldown				= Scripts.EmptyF;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsMount.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsMount.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
        
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "mount";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end
    
    BFButton.MountID		= MountID;
    BFButton.MountSpellID	= select(2, C_MountJournal.GetMountInfoByID(MountID));
    BFButton.MountName		= C_MountJournal.GetMountInfoByID(MountID);
    
    if (not InCombatLockdown()) then
        ScriptsMount.UpdateAttributes(BFButton);
        ScriptsMount.UpdateSecureCursor(BFButton);
    end
    ScriptsMount.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

end


--[[------------------------------------------------
    Set Mount Favorite
--------------------------------------------------]]
function Methods.SetActionMountFavorite(BFButton)
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "favoritemount") then
    
        BFButton.UpdateAction				= Scripts.EmptyF;
        BFButton.FullUpdate					= ScriptsMountFavorite.FullUpdate;
        BFButton.UpdateIcon					= ScriptsMountFavorite.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsMountFavorite.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsMountFavorite.UpdateUsable;
        BFButton.UpdateCooldown				= Scripts.EmptyF;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsMountFavorite.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsMountFavorite.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "favoritemount";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
        BFButton.MountID		= C.SUMMON_RANDOM_FAVORITE_MOUNT_ID;
        BFButton.MountSpellID	= C.SUMMON_RANDOM_FAVORITE_MOUNT_SPELL;
        BFButton.MountName		= GetSpellInfo(BFButton.MountSpellID);
        --if (not IsAddOnLoaded("Blizzard_Collections")) then
        --	LoadAddOn("Blizzard_Collections");
        --end

        Core.ResetDisplayState(BFButton);

    end
    
    if (not InCombatLockdown()) then
        ScriptsMountFavorite.UpdateAttributes(BFButton);
        ScriptsMountFavorite.UpdateSecureCursor(BFButton);
    end
    ScriptsMountFavorite.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

    
end


--[[------------------------------------------------
    Set Battle Pet
--------------------------------------------------]]
function Methods.SetActionBattlePet(BFButton, BattlePetGUID)
    
    if (not BattlePetGUID) then
        return Methods.SetActionEmpty(BFButton);
    end

    if (BattlePetGUID == C.SUMMON_RANDOM_FAVORITE_PET_GUID) then
        -- It's the random favorite button, use a different code path
        return Methods.SetActionBattlePetFavorite(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "battlepet") then
    
        BFButton.UpdateAction				= ScriptsBattlePet.UpdateAction;
        BFButton.FullUpdate					= ScriptsBattlePet.FullUpdate;
        BFButton.UpdateIcon					= ScriptsBattlePet.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsBattlePet.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsBattlePet.UpdateUsable;
        BFButton.UpdateCooldown				= ScriptsBattlePet.UpdateCooldown;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsBattlePet.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsBattlePet.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "battlepet";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end
    
    local speciesID, customName, level, xp, maxXp, displayID, isFavorite, name, icon = C_PetJournal.GetPetInfoByPetID(BattlePetGUID);
    BFButton.BattlePetGUID	= BattlePetGUID;
    BFButton.BattlePetName = customName or name;
    BFButton.IconTexture = icon;

    if (not InCombatLockdown()) then
        ScriptsBattlePet.UpdateAttributes(BFButton);
        ScriptsBattlePet.UpdateSecureCursor(BFButton);
    end
    ScriptsBattlePet.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);

end


--[[------------------------------------------------
    Set Battle Pet Favorite
--------------------------------------------------]]
function Methods.SetActionBattlePetFavorite(BFButton)
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "favoritebattlepet") then
    
        BFButton.UpdateAction				= Scripts.EmptyF;
        BFButton.FullUpdate					= ScriptsBattlePetFavorite.FullUpdate;
        BFButton.UpdateIcon					= ScriptsBattlePetFavorite.UpdateIcon;
        BFButton.UpdateChecked				= ScriptsBattlePetFavorite.UpdateChecked;
        BFButton.UpdateUsable				= ScriptsBattlePetFavorite.UpdateUsable;
        BFButton.UpdateCooldown				= ScriptsBattlePetFavorite.UpdateCooldown;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsBattlePetFavorite.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsBattlePetFavorite.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "favoritebattlepet";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);

        BFButton.IconTexture = GetSpellTexture(C.SUMMON_RANDOM_FAVORITE_PET_SPELL);



    end
    
    if (not InCombatLockdown()) then
        ScriptsBattlePetFavorite.UpdateAttributes(BFButton);
        ScriptsBattlePetFavorite.UpdateSecureCursor(BFButton);
    end
    ScriptsBattlePetFavorite.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);   
end


--[[------------------------------------------------
    Set Flyout
--------------------------------------------------]]
function Methods.SetActionFlyout(BFButton, FlyoutID)
    
    if (not FlyoutID) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "flyout") then
    
        BFButton.UpdateAction				= ScriptsFlyout.UpdateAction;	
        BFButton.FullUpdate					= ScriptsFlyout.FullUpdate;
        BFButton.UpdateIcon					= Scripts.EmptyF;
        BFButton.UpdateChecked				= Scripts.EmptyChecked;
        BFButton.UpdateUsable				= Scripts.EmptyF;
        BFButton.UpdateCooldown				= Scripts.EmptyF;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= Scripts.EmptyF;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsFlyout.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsFlyout.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;	
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "flyout";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end
    
    BFButton.FlyoutID = FlyoutID;
    BFButton.FlyoutName, BFButton.FlyoutDescription = GetFlyoutInfo(BFButton.FlyoutID);
    BFButton.IconTexture = select(2, Util.GetFlyoutIndexAndTexture(BFButton.FlyoutID));

    if (not InCombatLockdown()) then
        ScriptsFlyout.UpdateAttributes(BFButton);
        ScriptsFlyout.UpdateSecureCursor(BFButton);
    end
    ScriptsFlyout.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);   
end


--[[------------------------------------------------
    Set EquipmentSet
--------------------------------------------------]]
function Methods.SetActionEquipmentSet(BFButton, EquipmentSetName, EquipmentSetID)

    if (not EquipmentSetName) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    if (BFButton.Type ~= "equipmentset") then
    
        BFButton.UpdateAction				= ScriptsEquipmentSet.UpdateAction;
        BFButton.FullUpdate					= ScriptsEquipmentSet.FullUpdate;
        BFButton.UpdateIcon					= Scripts.EmptyF;
        BFButton.UpdateChecked				= Scripts.EmptyChecked;
        BFButton.UpdateUsable				= ScriptsEquipmentSet.UpdateUsable;
        BFButton.UpdateCooldown				= Scripts.EmptyF;
        BFButton.UpdateEquipped				= Scripts.EmptyF;
        BFButton.UpdateText					= ScriptsEquipmentSet.UpdateText;
        BFButton.UpdateGlow					= Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsEquipmentSet.GetCursor;
        BFButton.UpdateFlashRegistration	= Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= Scripts.EmptyF;
        BFButton.CheckRange					= Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsEquipmentSet.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "equipmentset";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    end

    BFButton.EquipmentSetName = EquipmentSetName;
    EquipmentSetID = EquipmentSetID or C_EquipmentSet.GetEquipmentSetID(EquipmentSetName);
    BFButton.EquipmentSetID = EquipmentSetID;
    local _, IconTexture = C_EquipmentSet.GetEquipmentSetInfo(BFButton.EquipmentSetID);
    BFButton.IconTexture = IconTexture;

    if (not InCombatLockdown()) then
        ScriptsEquipmentSet.UpdateAttributes(BFButton);
        ScriptsEquipmentSet.UpdateSecureCursor(BFButton);
    end
    ScriptsEquipmentSet.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);  
    
end


--[[------------------------------------------------
    Set Custom
--------------------------------------------------]]
function Methods.SetActionCustom(BFButton, CustomName, CustomData)
    local CustomAction = CustomActionRegistry.GetCustomAction(CustomName);
    if (not CustomAction) then
        return Methods.SetActionEmpty(BFButton);
    end
    local OldAction = BFButton.OldAction or Util.DeepCopy(BFButton.Action);
    BFButton.OldAction = nil;
    --It will probably always be necessary to always set the scripts for custom action swapping
    --if (BFButton.Type ~= "custom") then 
    
        BFButton.UpdateAction				= ScriptsCustom.UpdateAction;
        BFButton.FullUpdate					= ScriptsCustom.FullUpdate;
        BFButton.UpdateIcon					= ScriptsCustom.UpdateIcon;
        BFButton.UpdateChecked				= CustomAction.IsChecked 	and ScriptsCustom.UpdateChecked or Scripts.EmptyF;
        BFButton.UpdateUsable				= CustomAction.IsUsable 	and ScriptsCustom.UpdateUsable or Scripts.EmptyF;
        BFButton.UpdateCooldown				= CustomAction.GetCooldown 	and ScriptsCustom.UpdateCooldown or Scripts.EmptyF;
        BFButton.UpdateEquipped				= CustomAction.IsEquipped 	and ScriptsCustom.UpdateEquipped or Scripts.EmptyF;
        BFButton.UpdateText					= CustomAction.GetText 		and ScriptsCustom.UpdateText or Scripts.EmptyF;
        BFButton.UpdateGlow					= CustomAction.IsGlowing 	and ScriptsCustom.UpdateGlow or Scripts.EmptyF;
        BFButton.UpdateShine				= Scripts.EmptyF;
        BFButton.UpdateMacro				= Scripts.EmptyF;
        BFButton.GetCursor					= ScriptsCustom.GetCursor;
        BFButton.UpdateFlashRegistration	= CustomAction.IsFlashing and ScriptsCustom.UpdateFlashRegistration or Scripts.EmptyF;
        BFButton.UpdateRangeRegistration	= CustomAction.IsInRange and ScriptsCustom.UpdateRangeRegistration or Scripts.EmptyF;
        BFButton.CheckRange					= CustomAction.IsInRange and ScriptsCustom.CheckRange or Scripts.EmptyF;
        BFButton.ABW.UpdateTooltip			= ScriptsCustom.UpdateTooltip;
        BFButton.SwapActionWithButtonAction	= Scripts.SwapActionWithButtonAction;
    
        Core.ScrubActionValues(BFButton);
        Core.ScrubActionAttributeValues(BFButton);
        Core.ScrubIconTexCoords(BFButton);
        BFButton.Type = "custom";
        if (not InCombatLockdown()) then
            Core.UpdateButtonShowHide(BFButton);
        end
        --Core.UpdateOnCombatHideRegistration(BFButton);
        
    --end

    BFButton.CustomName = CustomName;
    BFButton.CustomData = CustomData;
    BFButton.CustomAction = CustomAction;

    if (not InCombatLockdown()) then
        ScriptsCustom.UpdateAttributes(BFButton);
        ScriptsCustom.UpdateSecureCursor(BFButton);
    end
    ScriptsCustom.UpdateActionTable(BFButton);

    Core.ResetDisplayState(BFButton);
    BFButton:FullUpdate();

    Core.ReportEvent(BFButton, C.EVENT_SETACTION, BFButton.Action, OldAction);  
    
end
