--[[
    Author: Alternator (Massiner of Nathrezim)
    Date:	2014

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local ScriptsBattlePet = Engine.ScriptsBattlePet;

local C = Engine.Constants;
local Core = Engine.Core;
local Util = Engine.Util;


--[[------------------------------------------------
    UpdateAction
--------------------------------------------------]]
function ScriptsBattlePet.UpdateAction(BFButton)
    if (Type and Type ~= "battlepet") then
        return;
    end
    local speciesID, customName, level, xp, maxXp, displayID, isFavorite
        , name, icon = C_PetJournal.GetPetInfoByPetID(BFButton.BattlePetGUID);
        
    BFButton.IconTexture = icon;
    if (BFButton.BattlePetName ~= customName or name) then
        BFButton.BattlePetName = customName or name;

        ScriptsBattlePet.UpdateActionTable(BFButton);
        Core.ReportEvent(BFButton, C.EVENT_UPDATEACTION, BFButton.Action);
    end
end

function ScriptsBattlePet.UpdateAttributes(BFButton)
    BFButton.ABW:SetAttribute("type", "macro");
    BFButton.ABW:SetAttribute("macrotext", format("/summonpet %s", BFButton.BattlePetGUID));
end
function ScriptsBattlePet.UpdateSecureCursor(BFButton)
    BFButton.ABW:SetAttribute("CursorKind", nil);
    BFButton.ABW:SetAttribute("CursorTarget", nil);
end
function ScriptsBattlePet.UpdateActionTable(BFButton)
    local Action = BFButton.Action;
    Util.ClearTable(Action);
    Action.Type = "battlepet";
    Action.BattlePetGUID = BFButton.BattlePetGUID;
    Action.BattlePetName = BFButton.BattlePetName;
end

--[[------------------------------------------------
    Full Update
--------------------------------------------------]]
function ScriptsBattlePet.FullUpdate(BFButton)
    BFButton.ABWIcon:SetTexture(BFButton.IconTexture);
    BFButton:UpdateChecked();
    BFButton:UpdateUsable();
    BFButton:UpdateCooldown();
    
    if (GetMouseFocus() == BFButton.ABW) then
        BFButton.ABW:UpdateTooltip();
    end
end


--[[------------------------------------------------
    Icon
--------------------------------------------------]]
function ScriptsBattlePet.UpdateIcon(BFButton)

end


--[[------------------------------------------------
    Checked
--------------------------------------------------]]
function ScriptsBattlePet.UpdateChecked(BFButton)
    if (BFButton.BattlePetGUID == C_PetJournal.GetSummonedPetGUID()) then
        BFButton.ABW:SetChecked(true);
    else
        BFButton.ABW:SetChecked(false);
    end
end


--[[------------------------------------------------
    Usable
--------------------------------------------------]]
function ScriptsBattlePet.UpdateUsable(BFButton)
    Core.UpdateButtonUsable(BFButton, C_PetJournal.PetIsSummonable(BFButton.BattlePetGUID));
end


--[[------------------------------------------------
    Cooldown
--------------------------------------------------]]
function ScriptsBattlePet.UpdateCooldown(BFButton)
    CooldownFrame_Set(BFButton.ABWCooldown, C_PetJournal.GetSummonBattlePetCooldown());
end


--[[------------------------------------------------
    Tooltip	(Copied from how the Pet Journal Shows the Tooltip)
--------------------------------------------------]]
function ScriptsBattlePet.UpdateTooltip(ABW)
    local BFButton = ABW.BFButton;
    GameTooltip_SetDefaultAnchor(GameTooltip, ABW);

    --[[ We can replace this now with a blizz function to show battlepet tooltips
    if ( BFButton.BattlePetName and BFButton.BattlePetName ~= "" ) then
        GameTooltip:SetText(BFButton.BattlePetName, 1, 1, 1);
        GameTooltip:AddLine(SPELL_CAST_TIME_INSTANT, 1, 1, 1, true);
        GameTooltip:AddLine(string.format(BATTLE_PET_TOOLTIP_SUMMON, BFButton.BattlePetSpeciesName), nil, nil, nil, true);
        GameTooltip:Show();
    end
    ]]
    GameTooltip:SetCompanionPet(ABW.BFButton.BattlePetGUID);
end


--[[------------------------------------------------
    Cursor
--------------------------------------------------]]
function ScriptsBattlePet.GetCursor(BFButton)
    return "battlepet", BFButton.BattlePetGUID, nil, nil;
end

