--[[
    Author: Alternator (Massiner of Nathrezim)
    Date:	2020
    
    Scripts for Custom Actions

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local ScriptsCustom = Engine.ScriptsCustom;

local Core = Engine.Core;
local Util = Engine.Util;
local C = Engine.Constants;


--[[------------------------------------------------
    UpdateAction		
--------------------------------------------------]]
function ScriptsCustom.UpdateAction(BFButton, Type)
    if (not Type or Type == "custom") then
        ScriptsCustom.UpdateAttributes(BFButton);
    end
end

function ScriptsCustom.UpdateAttributes(BFButton)
    local Type, ActionAttribute, ActionValue = BFButton.CustomAction.GetAction(BFButton.ABW, BFButton.CustomData);
    BFButton.ABW:SetAttribute("type", Type);
    BFButton.ABW:SetAttribute(ActionAttribute, ActionValue);
end
function ScriptsCustom.UpdateSecureCursor(BFButton)
    BFButton.ABW:SetAttribute("CursorKind", nil);
    BFButton.ABW:SetAttribute("CursorTarget", nil);
end
function ScriptsCustom.UpdateActionTable(BFButton)
    local Action = BFButton.Action;
    Util.ClearTable(Action);
    Action.Type = "custom";
    Action.CustomName = BFButton.CustomName;
    Action.CustomData = BFButton.CustomData;
end


--[[------------------------------------------------
    Full Update
--------------------------------------------------]]
function ScriptsCustom.FullUpdate(BFButton)
    BFButton:UpdateIcon();
    BFButton:UpdateChecked();
    BFButton:UpdateUsable();
    BFButton:UpdateCooldown();
    BFButton:UpdateText();
    BFButton:UpdateGlow();
    BFButton:UpdateFlashRegistration();
    BFButton:UpdateRangeRegistration();

    if (GetMouseFocus() == BFButton.ABW) then
        BFButton.ABW:UpdateTooltip();
    end
end


--[[------------------------------------------------
    Icon
--------------------------------------------------]]
function ScriptsCustom.UpdateIcon(BFButton)
    local Icon, left, right, top, bottom = BFButton.CustomAction.GetIcon(BFButton.ABW, BFButton.CustomData);
    if (left) then
        BFButton.ABWIcon:SetTexCoord(left, right, top, bottom);
    end
    BFButton.ABWIcon:SetTexture(Icon);
end


--[[------------------------------------------------
    Checked
--------------------------------------------------]]
function ScriptsCustom.UpdateChecked(BFButton)
    BFButton.ABW:SetChecked(BFButton.CustomAction.IsChecked(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Usable
--------------------------------------------------]]
function ScriptsCustom.UpdateUsable(BFButton)
    Core.UpdateButtonUsable(BFButton, BFButton.CustomAction.IsUsable(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Cooldown
--------------------------------------------------]]
function ScriptsCustom.UpdateCooldown(BFButton)
    CooldownFrame_Set(BFButton.ABWCooldown, BFButton.CustomAction.GetCooldown(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Text
--------------------------------------------------]]
function ScriptsCustom.UpdateText(BFButton)
    BFButton.ABWName:SetText(BFButton.CustomAction.GetText(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Glow
--------------------------------------------------]]
function ScriptsCustom.UpdateGlow(BFButton)
    if (BFButton.CustomAction.IsGlowing(BFButton.ABW, BFButton.CustomData)) then
        ActionButton_ShowOverlayGlow(BFButton.ABW);
    else
        ActionButton_HideOverlayGlow(BFButton.ABW);
    end
end


--[[------------------------------------------------
    Tooltip
--------------------------------------------------]]
function ScriptsCustom.UpdateTooltip(ABW)
    GameTooltip_SetDefaultAnchor(GameTooltip, ABW);
    ABW.BFButton.CustomAction.UpdateTooltip(ABW, ABW.BFButton.CustomData);
end


--[[------------------------------------------------
    Flash	(registers and Deregisters for flash)
    Notes:
        * WoW v7
            * IsAttackSpell does not work with SpellID, so we need to use SpellFullName here
            * Tidy up
                * Removed check for AutoRepeatSpells (I dont believe they are still a thing)
--------------------------------------------------]]
function ScriptsCustom.UpdateFlashRegistration(BFButton)
    local IsFlashing = BFButton.CustomAction.IsFlashing(BFButton.ABW, BFButton.CustomData);
    Core.UpdateButtonFlashRegistration(BFButton, IsFlashing);
end


--[[------------------------------------------------
    Range Registration
    Notes:
        * WoW v7
            * IsSpellInRange does not work with SpellID, so we need to use SpellFullName here
--------------------------------------------------]]
function ScriptsCustom.UpdateRangeRegistration(BFButton)
    Core.UpdateButtonRangeRegistration(BFButton, BFButton.CustomAction.IsInRange(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Range Check
    Notes:
        * WoW v7
            * IsSpellInRange does not work with SpellID, so we need to use SpellFullName here
--------------------------------------------------]]
function ScriptsCustom.CheckRange(BFButton)
    Core.UpdateButtonRangeIndicator(BFButton, BFButton.CustomAction.IsInRange(BFButton.ABW, BFButton.CustomData));
end


--[[------------------------------------------------
    Cursor
--------------------------------------------------]]
function ScriptsCustom.GetCursor(BFButton)
    return "custom", BFButton.CustomName, BFButton.CustomData, nil, BFButton.CustomAction.GetIcon(BFButton.ABW, BFButton.CustomData);
end

