--[[
    Author: Alternator (Massiner of Nathrezim)
    Date:	2014

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local ScriptsEquipmentSet = Engine.ScriptsEquipmentSet;

local C = Engine.Constants;
local Core = Engine.Core;
local Util = Engine.Util;
local Methods = Engine.Methods;


--[[------------------------------------------------
    UpdateAction
--------------------------------------------------]]
function ScriptsEquipmentSet.UpdateAction(BFButton, Type)
    if (Type and Type ~= "equipmentset") then
        return;
    end
    local EquipmentSetName, IconTexture = C_EquipmentSet.GetEquipmentSetInfo(BFButton.EquipmentSetID);
    if (EquipmentSetName == nil) then
        -- This equip set is gone so clear it from the button
        return Methods.SetActionEmpty(BFButton);
    end

    BFButton.IconTexture = IconTexture;
    if (BFButton.EquipmentSetName ~= EquipmentSetName) then
        BFButton.EquipmentSetName = EquipmentSetName;

        ScriptsEquipmentSet.UpdateAttributes(BFButton);
        ScriptsEquipmentSet.UpdateSecureCursor(BFButton);
        ScriptsEquipmentSet.UpdateActionTable(BFButton);
        Core.ReportEvent(BFButton, C.EVENT_UPDATEACTION, BFButton.Action);
    end
end

function ScriptsEquipmentSet.UpdateAttributes(BFButton)
    BFButton.ABW:SetAttribute("type", "equipmentset");
    BFButton.ABW:SetAttribute("equipmentset", BFButton.EquipmentSetName);
end
function ScriptsEquipmentSet.UpdateSecureCursor(BFButton)
    BFButton.ABW:SetAttribute("CursorKind", "equipmentset");
    BFButton.ABW:SetAttribute("CursorTarget", BFButton.EquipmentSetID);
end
function ScriptsEquipmentSet.UpdateActionTable(BFButton)
    local Action = BFButton.Action;
    Util.ClearTable(Action);
    Action.Type = "equipmentset";
    Action.EquipmentSetName = BFButton.EquipmentSetName;
    Action.EquipmentSetID = BFButton.EquipmentSetID;
end

--[[------------------------------------------------
    Full Update
--------------------------------------------------]]
function ScriptsEquipmentSet.FullUpdate(BFButton)

    BFButton.ABWIcon:SetTexture(BFButton.IconTexture);
    BFButton:UpdateChecked();
    BFButton:UpdateText();
    
    if (GetMouseFocus() == BFButton.ABW) then
        BFButton.ABW:UpdateTooltip();
    end
end


--[[------------------------------------------------
    Checked
--------------------------------------------------]]
function ScriptsEquipmentSet.UpdateChecked(BFButton)
    BFButton.ABW:SetChecked(false);
end


--[[------------------------------------------------
    Usable
--------------------------------------------------]]
function ScriptsEquipmentSet.UpdateUsable(BFButton)
    Core.UpdateButtonUsable(BFButton, true);
end


--[[------------------------------------------------
    Text
--------------------------------------------------]]
function ScriptsEquipmentSet.UpdateText(BFButton)
    BFButton.ABWName:SetText(BFButton.EquipmentSetName);

end


--[[------------------------------------------------
    Tooltip
--------------------------------------------------]]
function ScriptsEquipmentSet.UpdateTooltip(ABW)
    GameTooltip_SetDefaultAnchor(GameTooltip, ABW);
    GameTooltip:SetEquipmentSet(ABW.BFButton.EquipmentSetName);
end


--[[------------------------------------------------
    Cursor
--------------------------------------------------]]
function ScriptsEquipmentSet.GetCursor(BFButton)
    return "equipmentset", BFButton.EquipmentSetID, nil, nil;
end

