--[[
    Author: Alternator (Massiner of Nathrezim)
    Date:	2014

]]

local AddonName, AddonTable = ...;
local Engine = AddonTable.ButtonEngine;
local SecureActionSwap = Engine.SecureActionSwap;
local Util = Engine.Util;

local S = Engine.Settings;
local C = Engine.Constants;
local Core = Engine.Core;
local Cursor = Engine.Cursor;
local FlyoutUI = Engine.FlyoutUI;


local SecureActionSwapHandler = CreateFrame("FRAME", nil, nil, "SecureHandlerBaseTemplate");
SecureActionSwap.SecureActionSwapHandler = SecureActionSwapHandler;

SecureActionSwapHandler:Execute(format([[SpellMap = newtable(); FavoriteMountID = %i;]], C.SUMMON_RANDOM_FAVORITE_MOUNT_ID));


-- Always clear the cursor before pickup, incase the swap is with the same action (which would cause the cursor to unset, but not pickup the new action)
    -- Pickup for now is disabled, although the code is in place to pickup the action, from a usability standpoint it is cumbersome as we cannot then place it on a BF Button, so most likely
    -- the user will just want to dump the action off the cursor
    -- There is no way to reenter drag mode from performing a drag action to a button, and OnClick isn't allowed to see contents of the Cursor (we could track the contents in Secure env
    -- however we can't detect if the user dumps the action)
    -- the one exception being we "theoretically" can track custom actions for use with BF Buttons OnClick... If that does pan out, it might be an option to shadow the cursor for a standard WoW Action
SecureActionSwap.SecureOnReceiveDragSnippet = [[
    -- If we are not in combat have the unsecure handler run, it is better
    if (PlayerInCombat()) then
        if (self:GetAttribute("FullLock") or self:GetAttribute("Source")) then
            return;
        end

        local Type, ValueAttr, Value, NewKind, NewTarget = owner:RunAttribute("SetAction", kind, value, ...);

        -- Store return cursor
        local RKind         = self:GetAttribute("CursorKind");
        local RTarget       = self:GetAttribute("CursorTarget");
        if (Type) then

            -- Scrub the attributes
            self:SetAttribute("type"		, nil);
            self:SetAttribute("type2"	    , nil);
            self:SetAttribute("spell"	    , nil);
            self:SetAttribute("item"		, nil);
            self:SetAttribute("macro"	    , nil);
            self:SetAttribute("macrotext"   , nil);
            self:SetAttribute("macrotext2"  , nil);
            self:SetAttribute("action"	    , nil);
            self:SetAttribute("id"		    , nil);

            -- Set the secure cursor info
            self:SetAttribute("CursorKind"      , NewKind);
            self:SetAttribute("CursorTarget"    , NewTarget);

            -- Set the attributes for the action
            self:SetAttribute("type", Type);
            self:SetAttribute(ValueAttr, Value);

            -- Perform unsecure tasks to finish setting up action
            owner:CallMethod("SetActionFromCursor", self:GetName(), kind, value, ...);

            return "clear", RKind, RTarget;
        end
    end
]];
SecureActionSwap.SecureOnDragStartSnippet = [[
    -- If we are not in combat have the unsecure handler run, it is better
    if (PlayerInCombat()) then
        if (self:GetAttribute("FullLock")) then
            return;
        end
        if (self:GetAttribute("Locked") and not IsShiftKeyDown()) then
            return;
        end

        -- Store return cursor
        local RKind         = self:GetAttribute("CursorKind");
        local RTarget       = self:GetAttribute("CursorTarget");
        local Type, ValueAttr, Value, NewKind, NewTarget = owner:RunAttribute("SetAction", kind, value, ...);

        if (not self:GetAttribute("Source") and Type) then

            -- Scrub the attributes
            self:SetAttribute("type"		, nil);
            self:SetAttribute("type2"	    , nil);
            self:SetAttribute("spell"	    , nil);
            self:SetAttribute("item"		, nil);
            self:SetAttribute("macro"	    , nil);
            self:SetAttribute("macrotext"   , nil);
            self:SetAttribute("macrotext2"  , nil);
            self:SetAttribute("action"	    , nil);
            self:SetAttribute("id"		    , nil);

            -- Set the secure cursor info
            self:SetAttribute("CursorKind"      , NewKind);
            self:SetAttribute("CursorTarget"    , NewTarget);

            -- Set the attributes for the action
            if (Type ~= "empty") then
                self:SetAttribute("type", Type);
                self:SetAttribute(ValueAttr, Value);
            end

            -- Perform unsecure tasks to finish setting up action
            owner:CallMethod("SetActionFromCursor", self:GetName(), kind, value, ...);
        end
        return "clear", RKind, RTarget;
    end
]];

local SecureActionFromCursorSnippet = [[
    local Type, Data, Value, SubValue, SubSubValue = ...; -- SubSubValue seems to be needed for Sinister Strike, even though SubValue is the SpellID, the players Spellbook reports it as the SubSubValue 1752 instead of 193315
    if (Type == "spell") then
        if (SpellMap[SubValue]) then
            return "spell", "spell", SpellMap[SubValue], "spell", SubValue;
        elseif (SpellMap[SubSubValue or 0]) then
            return "spell", "spell", SpellMap[SubSubValue], "spell", SubValue;
        end
    elseif (Type == "item") then
        return "item", "item", strmatch(Value, "%[(.+)%]"), "item", Data;
    elseif (Type == "macro") then
        return "macro", "macro", Data, "macro", Data;
    elseif (Type == "mount") then
        if (Data == FavoriteMountID) then
            return "macro", "macrotext", "/script C_MountJournal.SummonByID(0);";
        else
            return "macro", "macrotext", format("/script C_MountJournal.SummonByID(%i);", Data); -- Pickup of mount via secure code is broken in WoW 9
        end
    elseif (Type == "battlepet") then
        return "macro", "macrotext", format("/summonpet %s", Data); -- Pickup of pet via secure code is broken in WoW 9
    elseif (Type == "flyout") then
        return "flyout", "spell", Data;     -- There is no secure code path to pickup flyouts
    elseif (Type == "equipmentset") then
        return "equipmentset", "equipmentset", Data, "equipmentset", Data;
    else
        return "empty";
    end
]];
SecureActionSwapHandler:SetAttribute("SetAction", SecureActionFromCursorSnippet);

local UpdateSecureSpellMapSnippet = [[
    if ( owner:GetAttribute("Known") or not SpellMap[owner:GetAttribute("SpellID")] ) then
        SpellMap[owner:GetAttribute("SpellID")] = owner:GetAttribute("AttrValue");
    end
]];


function SecureActionSwapHandler:SetActionFromCursor(ABWName, Type, Data, Subvalue, SubSubvalue)
    Cursor.SetActionFromCursorValues(_G[ABWName].BFButton, Type, Data, Subvalue, SubSubvalue, true);
end

--[[------------------------------------------------
    UpdateSecureSpellMap
        WoW v9.0.2
            * Provides mapping from SpellID to the correct value to use for the "spell" attribute
            * The first three tabs contain spells we can currently use

            * loop explanation
                * SPELL
                    * SPELL indicates a "known" spell
                    * FUTURESPELL is a currently unknown spell
                    * For spells that act as an override always use the base spell info as the AttrValue (the jury is out on SpellID... but SpellFullName is the majority of cases and is the reliable method)
                    * We will update the map with known spell info, but we will only add unknown spells we will not update - this is to avoid possible issues around override type spells that may not report
                      their nature correctly when unknown
                * FLYOUT
                    * We need to check IsKnown for both the flyout and it's slots

                * Professions
                    * We need to capture professions and use the SpellID's for those
--------------------------------------------------]]
function SecureActionSwap.UpdateSecureSpellMap()
    if (InCombatLockdown()) then
        Core.LeaveCombatQueueUpFunction(SecureActionSwap.UpdateSecureSpellMap);
        return;
    end
    for tab = 1, 3 do
        local TabStart, TabCount = select(3, GetSpellTabInfo(tab));
        for i = TabStart + 1, TabStart + TabCount do
            local Type, SpellID = GetSpellBookItemInfo(i, BOOKTYPE_SPELL);

            if ((Type == "SPELL" or Type == "FUTURESPELL") and SpellID) then
                SecureActionSwapHandler:SetAttribute("Known", Type == "SPELL");
                SecureActionSwapHandler:SetAttribute("SpellID", SpellID);
                SpellID = FindBaseSpellByID(SpellID);
                if (Util.IsSpellCastByID(SpellID)) then
                    SecureActionSwapHandler:SetAttribute("AttrValue", SpellID);
                else
                    SecureActionSwapHandler:SetAttribute("AttrValue", Util.SpellFullName(SpellID));
                end
                SecureActionSwapHandler:Execute(UpdateSecureSpellMapSnippet);

            elseif (Type == "FLYOUT") then
                local FlyoutID = SpellID;
                local _, _, numSlots, isKnown = GetFlyoutInfo(FlyoutID);

                for k = 1, numSlots do
                    local SpellID, overrideSpellID, isKnown = GetFlyoutSlotInfo(FlyoutID, k);
                    SecureActionSwapHandler:SetAttribute("Known", isKnown);
                    if (Util.IsSpellCastByID(SpellID)) then
                        SecureActionSwapHandler:SetAttribute("AttrValue", SpellID);
                    else
                        SecureActionSwapHandler:SetAttribute("AttrValue", Util.SpellFullName(SpellID));
                    end
                    SecureActionSwapHandler:SetAttribute("SpellID", SpellID);
                    SecureActionSwapHandler:Execute(UpdateSecureSpellMapSnippet);
                    SecureActionSwapHandler:SetAttribute("SpellID", overrideSpellID);
                    SecureActionSwapHandler:Execute(UpdateSecureSpellMapSnippet);
                end
            end
        end
    end

    local prof1, prof2, arch, fish, cook = GetProfessions();
    local profs = {[1] = prof1, [2] = prof2};

    for k, v in ipairs(profs) do
        local name, texture, rank, maxRank, numSpells, spellOffset, skillLine, rankModifier, specializationIndex, specializationOffset, skillLineName = GetProfessionInfo(v);
        for i = 1, numSpells do
            local _, SpellID = GetSpellBookItemInfo(spellOffset + i, BOOKTYPE_PROFESSION);
            SecureActionSwapHandler:SetAttribute("Known", true);
            SecureActionSwapHandler:SetAttribute("SpellID", SpellID);
            SecureActionSwapHandler:SetAttribute("AttrValue", SpellID);
            SecureActionSwapHandler:Execute(UpdateSecureSpellMapSnippet);
        end
    end

end
SecureActionSwap.UpdateSecureSpellMap();

